class BinaryHeap {
	
	static final int CAPACITY = 7;
	static int array[] = new int[CAPACITY + 1];
	static int currentSize;
	
	/**
	 * Inserts a new element into the heap. First put a hole into the end
	 *	of heap, and then percolate up.
	 */
	public static void insert(int x) {
		
		if (currentSize == CAPACITY) {
			System.err.println("Heap full");
			System.exit(1);
		}
		
		int hole = ++currentSize;
		for (; hole > 1 && x < array[hole / 2]; hole /= 2) {
			array[hole] = array[hole/2];
		}
		array[hole] = x;
	}
	
	/**
	 * Removes the smallest item from the heap. 
	 *	Heap order is preserved by percodalte down.
	 */
	public static int deleteMin() {
		
		if (currentSize == 0) {
			System.err.println("Heap empty");
			System.exit(1);
		}
		
		int minItem = array[1];
		
		array[1] = array[currentSize];
		array[currentSize] = 0;
		currentSize--;
		percolateDown(1);
		
		return minItem;
	}
	
	/**
	 * Percolates the hole down the heap.
	 */
	static void percolateDown(int hole) {
		
		int child;
		int tmp = array[hole];
		
		for (; hole * 2 <= currentSize; hole = child) {
			child = hole * 2;
			
			//if (array[child + 1] < array[child])
			if (child != currentSize && array[child + 1] < array[child])
				child++;
			if (array[child] < tmp)
				array[hole] = array[child];
			else
				break;
		}
		array[hole] = tmp;
	}
	
	/**
	 * Inserts the argument values into the heap.
	 * 	if heap property hold, call deleteMin() 
	 *	and check the property again.
	 */
	public static void main(String[] args) {
		
		for (int i = 0; i < args.length; i++) {
			insert(Integer.parseInt(args[i]));
		}
		
		if (isPropertyHold(1)) {
			print();
			System.out.println("deleteMin() = " + deleteMin());
			print();
			assert(isPropertyHold(1));
		}
	}
	
	static boolean isPropertyHold(int x) {
		
		int child = x * 2;
		
		if (child > currentSize)
			return true;
		if (array[x] > array[child]) 
			return false;
		if (child != currentSize && array[x] > array[child + 1])
			return false;
		
		return isPropertyHold(child) && isPropertyHold(child + 1);
	}
	
	static void print() {
		System.out.print("Current Heap = ");
		print(1);
		System.out.println();
	}
	
	static void print(int x) {
		
		int child = x * 2;
		
		if (x > currentSize) {
			return;
		}
		if (child <= currentSize) {
			System.out.print("(");
			print(child);
			System.out.print(") ");
		}
		System.out.print(array[x]);
		if (child + 1 <= currentSize) {
			System.out.print(" (");
			print(child + 1);
			System.out.print(")");
		}
	}
	
	static void ok() {
		while (true) {}
	}
}
